<?php

if (IN_serendipity !== true) {
    die ("Don't hack!");
}


// Probe for a language include with constants. Still include defines later on, if some constants were missing
$probelang = dirname(__FILE__) . '/' . $serendipity['charset'] . 'lang_' . $serendipity['lang'] . '.inc.php';
if (file_exists($probelang)) {
    include $probelang;
}

include dirname(__FILE__) . '/lang_en.inc.php';

class serendipity_plugin_buzz extends serendipity_plugin {
    var $title = PLUGIN_BUZZ_NAME;

    function introspect(&$propbag) {
        global $serendipity;

        $propbag->add('name',          PLUGIN_BUZZ_NAME);
        $propbag->add('description',   PLUGIN_BUZZ_DESC);
        $propbag->add('stackable',     true);
        $propbag->add('author',        'Malte Paskuda');
        $propbag->add('version',       '0.1');
        $propbag->add('requirements',  array(
            'serendipity' => '0.8',
            'php' => '5.0'
        ));
        $propbag->add('configuration', array('username', 'amount', 'blogfilter'));
        $propbag->add('groups', array('FRONTEND_EXTERNAL_SERVICES'));
    }

    function introspect_config_item($name, &$propbag) {
        switch($name) {

            case 'username':
                $propbag->add('type', 'string');
                $propbag->add('name', PLUGIN_BUZZ_USERNAME);
                $propbag->add('description', PLUGIN_BUZZ_USERNAME_DESC);
                $propbag->add('default', '');
                break;
            case 'amount':
                $propbag->add('type', 'string');
                $propbag->add('name', PLUGIN_BUZZ_AMOUNT);
                $propbag->add('description', PLUGIN_BUZZ_AMOUNT_DESC);
                $propbag->add('default', '5');
                break;
            case 'blogfilter':
                $propbag->add('type', 'boolean');
                $propbag->add('name', PLUGIN_BUZZ_BLOGFILTER);
                $propbag->add('description', PLUGIN_BUZZ_BLOGFILTER_DESC);
                $propbag->add('default', true);
                break;
            default:
                return false;
        }
        return true;
    }

    function generate_content(&$title) {
        global $serendipity;
        $title = $this->title;
        $username = $this->get_config('username', '');
        $amount = $this->get_config('amount', 5) ;
        $blogfilter = $this->get_config('blogfilter', true) ;
        
        if (empty($username)) {
            return false;
        }

        $feed = $this->getFeed($username);
        if ($blogfilter) {
            $feed = $this->filterOwn($feed);
        }
        $text = $this->filterFeed($feed, 'content');
        for ($i=0; $i<=$amount; $i++) {
            $smarty_items['buzzs'][$i]['text'] = $text->item($i)->textContent;
        }
        $smarty_items['homelink'] = $this->getHomeLink($feed);
        
        if (!is_object($serendipity['smarty'])) {
            serendipity_smarty_init();
        }
        $tpl = 'plugin_buzz.tpl';
        $serendipity['smarty']->assign_by_ref('buzz_plugin', $smarty_items);
        
        $content = $this->parseTemplate($tpl);
        echo $content;
        return true;
    }

    function getFeed($username) {
        $url = "http://buzz.googleapis.com/feeds/$username/public/posted";
        $feed = file_get_contents($url);
        if (empty($feed)) {
            //try it again with curl if fopen was forbidden
            if (function_exists('curl_init')) {
                $ch = curl_init($url);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                $feed = curl_exec($ch);
                curl_close($ch);
            } 
        }
        return $feed;
    }

    #Filter specific element from the atom-feed of Buzz
    #TODO: media:content
    #        <media:title , medium='video'
    #$feed      string      The feed itself
    #$filter    string      The element searching for
    #$activity  bool        Shall the activity stream also be searched?
    #
    #return:    Array       Textvalue of nodes
    function filterFeed($feed, $filter='title', $activity=false) {
        $dom = new DomDocument();
        $dom->loadXML($feed);
        #Filter activities, important because of double tagname (title, content)
        if(!$activity) {
            $activities = $dom->getElementsByTagNameNS('http://activitystrea.ms/spec/1.0/', 'object');
            foreach ($activities as $activity) {
                #going this strange way because a simple
                # foreach node {node->parentnode->removeChild(node)} don't work
                while($activity->hasChildNodes()) {
                    $activity->removeChild($activity->childNodes->item(0));
                }
            }
        }
        
        $elements = $dom->getElementsByTagName($filter);
        return $elements;
    }

    function getHomeLink($feed) {
        $authors = $this->filterFeed($feed, 'author');
        $author = $authors->item(0);
        while($author->hasChildNodes()) {
            if ($author->firstChild->nodeName == 'uri') {
                return $author->firstChild->textContent;
            }
            $author->removeChild($author->firstChild);
        }
        return false;
    }

    function filterOwn($feed) {
        global $serendipity;

        $blogtitle = $serendipity['blogTitle'];
        #needs testing: buzz changes the blogtitle
        $blogtitle = str_replace('-', ' ', $blogtitle);
        
        $dom = new DomDocument();
        $dom->loadXML($feed);
        
        $titles = $dom->getElementsByTagName('title');
        foreach ($titles as $title) {
            #we don't want to mess with double titles of activities
            if ($title->prefix == 'acitivity') {
                continue;
            }
            if (strpos($title->textContent, 'from '.$blogtitle) !== false) {
                $entry = $title->parentNode;
                while($entry->hasChildNodes()) {
                    $entry->removeChild($entry->childNodes->item(0));
                }
            }
        }
        return $dom->saveXML();
    }

    function debugMsg($msg) {
		global $serendipity;
		
		$this->debug_fp = @fopen ( $serendipity ['serendipityPath'] . 'templates_c/pluginname.log', 'a' );
		if (! $this->debug_fp) {
			return false;
		}
		
		if (empty ( $msg )) {
			fwrite ( $this->debug_fp, "failure \n" );
		} else {
			fwrite ( $this->debug_fp, print_r ( $msg, true ) );
		}
		fclose ( $this->debug_fp );
	}

}

/* vim: set sts=4 ts=4 expandtab : */
?>
